//    IPS Java Utils
//    (c) Copyright 2009-2011
//    Institute of Phonetics and Speech Processing,
//    Ludwig-Maximilians-University, Munich, Germany
//
//
//    This file is part of IPS Java Utils
//
//
//    IPS Java Utils is free software: you can redistribute it and/or modify
//    it under the terms of the GNU Lesser General Public License as published by
//    the Free Software Foundation, version 3 of the License.
//
//    IPS Java Utils is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public License
//    along with IPS Java Utils.  If not, see <http://www.gnu.org/licenses/>.

package ipsk.net;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;

/**
 * Generates absolute URLs from relative URLs in a given context.
 * This class works much like @see java.net.URL#Constructor(URL context,String spec) .
 * @author klausj
 *
 */
public class URLContext {

	private URL context;

	public URLContext() {
		super();
	}

	/**
	 * Creates an URL context.
	 * @param context the context
	 */
	public URLContext(URL context) {
		super();
		this.context = context;
	}

	/**
			 * Creates a URL by parsing the given URL within the specified context.
			 * If context is <code>null</code> the given URL is returned.
			 * If the URL is relative it is appended to the context.
			 * This method works much like the constructor @see java.net.URL#Constructor(URL context,String spec) ,
			 * but it treats Windows pathnames like "C:/foo" as absolute.   
			 * @param relURL the URL to transform
			 * @return absolute URL in the given context 
			 * @throws MalformedURLException
			 */
	public URL getContextURL(URL relURL) throws MalformedURLException {
		return URLContext.getContextURL(context, relURL);
	}
	/**
			 * Creates a URL by parsing the given spec within the specified context.
			 * If context is <code>null</code> an URL from the given spec is created. 
			 * This method works much like the constructor @see java.net.URL#Constructor(URL context,String spec) ,
			 * but it treats Windows pathnames like "C:/foo" as absolute.   
			 * @param spec URL specifcation
			 * @return absolute URL in the given context 
			 * @throws MalformedURLException
			 */
	public URL getContextURL(String spec) throws MalformedURLException {
		return URLContext.getContextURL(context, spec);
	}
	/**
				 * Creates a URL by parsing the given URL within the specified context.
				 * If context is <code>null</code> the given URL is returned.
				 * If the URL is relative it is appended to the context.
				 * This method works much like the constructor @see java.net.URL#Constructor(URL context,String spec) ,
				 * but it treats Windows pathnames like "C:/foo" as absolute.   
				 * @param contextURL the context
				 * @param relURL the URL to transform
				 * @return absolute URL in the given context 
				 * @throws MalformedURLException
				 */
	public static URL getContextURL(URL contextURL, URL relURL)
		throws MalformedURLException {
		if (contextURL == null)
			return relURL;
		if (relURL.getProtocol().equals("file")) {
			// the context constructor for URL does not recognize absolute windows paths (e.g. C:/foo)
			// if it does not begin with a "/"
			//URL tmpURL=new URL(recDirName);
			String dirPath = relURL.getPath();
			File tmpFile = new File(dirPath);
			if (!dirPath.startsWith("/") && tmpFile.isAbsolute()) {
				dirPath = "/".concat(dirPath);
			}
			return new URL(contextURL, "file:" + dirPath);
		} else {
			return new URL(contextURL, relURL.toExternalForm());
		}
	}

	/**
	 * Creates a URL by parsing the given spec within the specified context.
	 * If context is <code>null</code> an URL from the given spec is created. 
	 * This method works much like the constructor @see java.net.URL#Constructor(URL context,String spec) ,
	 * but it treats Windows pathnames like "C:/foo" as absolute.
	 * Note: the method accepts relative URLs with scheme part, which are not RFC conform.
	 * This URL's are used in Speechrecorder for project dir relative resources, e.g. file:project1_script.xml
	 * Use @see ipsk.net.URLContext#getContextURLStrict(URL context,String spec) for strict RFC behavoiur
	 * @param contextURL the context
	 * @param spec URL specifcation
	 * @return absolute URL in the given context 
	 * @throws MalformedURLException
	 */
	public static URL getContextURL(URL contextURL, String spec)
		throws MalformedURLException {
		 URI contextUri=null;
	        try {
	            contextUri = contextURL.toURI();
	        } catch (URISyntaxException e) {
	           throw new MalformedURLException(e.getMessage());
	        }
	    URI specURI=null;
		try {
			specURI = new URI(spec);
			if(specURI.isAbsolute()){
		        URL relURL = new URL(spec);
		        return URLContext.getContextURL(contextURL, relURL);
		    }
		} catch (URISyntaxException e1) {
			try {
				specURI = new URI(null,spec,null);
				
			} catch (URISyntaxException e) {
				throw new MalformedURLException(e.getMessage());
			}
			
		}
	    
	   
	    URI resURI=contextUri.resolve(specURI);
	    URL resURL=resURI.toURL();
	    return resURL;
		
	}
	
	/**
     * Creates a URL by parsing the given spec within the specified context.
     * If context is <code>null</code> an URL from the given spec is created. 
     * This method works much like the constructor @see java.net.URL#Constructor(URL context,String spec) ,
     * but it treats Windows pathnames like "C:/foo" as absolute.
     * @param contextURL the context
     * @param spec URL specifcation
     * @return absolute URL in the given context 
     * @throws MalformedURLException
     */
	public static URL getContextURLStrict(URL contextURL, String spec)
	throws MalformedURLException {
	    URI specURI=URI.create(spec);
	    if(specURI.isAbsolute()){
	        return specURI.toURL();
	    }else{
	        URI contextUri=null;
	        try {
	            contextUri = contextURL.toURI();
	        } catch (URISyntaxException e) {
	            throw new MalformedURLException(e.getMessage());
	        }
	        URI resURI=contextUri.resolve(spec);
	        URL resURL=resURI.toURL();
	        return resURL;
	    }

	}
    
	
	

	/**
	 * Get URL context.
	 * @return the context
	 */
	public URL getContext() {
		return context;
	}

	/**
	 * Set URL context.
	 * @param url context
	 */
	public void setContext(URL url) {
		context = url;
	}
	
	public static void main(String[] args){
	    try {
            File testFile=new File("/homes/klausj/test blank/testSSʊʊʙʘʧöüß.txt");
            testFile.mkdirs();
            testFile.createNewFile();
//            File testFile=new File("\\C:\\homes\\klausj\\test blank\\testäöüß.txt");
            System.out.println(testFile.toURL().toExternalForm());
            URI testFileUri=testFile.toURI();
            URL testFileUrl=testFileUri.toURL();
            
           
            URI decTestUri=testFileUrl.toURI();
            URL decTestUrl=decTestUri.toURL();
            String decTestFilePath=decTestUri.getPath();
            File decTestFile=new File(decTestFilePath);
            boolean testOk=testFile.equals(decTestFile);
            System.out.println("Test OK:"+testOk+" Exists:"+decTestFile.exists());
            
        } catch (MalformedURLException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (URISyntaxException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
	}
	

}
